package com.jjt.biz.controller;

import com.jjt.biz.domain.AlarmRecord;
import com.jjt.biz.service.IAlarmRecordService;
import com.jjt.common.annotation.Log;
import com.jjt.common.core.controller.BaseController;
import com.jjt.common.core.domain.AjaxResult;
import com.jjt.common.core.page.TableDataInfo;
import com.jjt.common.enums.BusinessType;
import com.jjt.common.utils.poi.ExcelUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 告警记录Controller
 *
 * @author jjt
 * @date 2024-08-08
 */
@Api(tags = "告警记录")
@RestController
@RequestMapping("/alarm/record")
public class AlarmRecordController extends BaseController {
    @Resource
    private IAlarmRecordService alarmRecordService;

    /**
     * 查询告警记录列表
     */
    @ApiOperation("查询告警记录列表")
    @PreAuthorize("@ss.hasPermi('alarm:record:list')")
    @GetMapping("/list")
    public TableDataInfo list(AlarmRecord alarmRecord) {
        startPage();
        List<AlarmRecord> list = alarmRecordService.selectAlarmRecordList(alarmRecord);
        return getDataTable(list);
    }

    /**
     * 导出告警记录列表
     */
    @ApiOperation("导出告警记录列表")
    @PreAuthorize("@ss.hasPermi('alarm:record:export')")
    @Log(title = "告警记录" , businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, AlarmRecord alarmRecord) {
        List<AlarmRecord> list = alarmRecordService.selectAlarmRecordList(alarmRecord);
        ExcelUtil<AlarmRecord> util = new ExcelUtil<AlarmRecord>(AlarmRecord.class);
        util.exportExcel(response, list, "告警记录数据");
    }

    /**
     * 获取告警记录详细信息
     */
    @ApiOperation("获取告警记录详细信息")
    @PreAuthorize("@ss.hasPermi('alarm:record:query')")
    @GetMapping(value = "/{alarmId}")
    public AjaxResult getInfo(@PathVariable("alarmId") Long alarmId) {
        return success(alarmRecordService.selectAlarmRecordByAlarmId(alarmId));
    }

    /**
     * 新增告警记录
     */
    @ApiOperation("新增告警记录")
    @PreAuthorize("@ss.hasPermi('alarm:record:add')")
    @Log(title = "告警记录" , businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody AlarmRecord alarmRecord) {
        return toAjax(alarmRecordService.insertAlarmRecord(alarmRecord));
    }

    /**
     * 修改告警记录
     */
    @ApiOperation("修改告警记录")
    @PreAuthorize("@ss.hasPermi('alarm:record:edit')")
    @Log(title = "告警记录" , businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody AlarmRecord alarmRecord) {
        return toAjax(alarmRecordService.updateAlarmRecord(alarmRecord));
    }

    /**
     * 删除告警记录
     */
    @ApiOperation("删除告警记录")
    @PreAuthorize("@ss.hasPermi('alarm:record:remove')")
    @Log(title = "告警记录" , businessType = BusinessType.DELETE)
    @DeleteMapping("/{alarmIds}")
    public AjaxResult remove(@PathVariable Long[] alarmIds) {
        return toAjax(alarmRecordService.deleteAlarmRecordByAlarmIds(alarmIds));
    }
}
