package com.jjt.biz.controller;

import com.jjt.biz.domain.BizModel;
import com.jjt.biz.domain.BizObj;
import com.jjt.biz.service.IBizModelService;
import com.jjt.common.annotation.Log;
import com.jjt.common.core.controller.BaseController;
import com.jjt.common.core.domain.AjaxResult;
import com.jjt.common.core.page.TableDataInfo;
import com.jjt.common.enums.BusinessType;
import com.jjt.common.utils.poi.ExcelUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 业务模型Controller
 *
 * @author jjt
 * @date 2024-08-26
 */
@Api(tags = "业务模型")
@RestController
@RequestMapping("/hl/bm")
public class BizModelController extends BaseController {
    @Resource
    private IBizModelService bizModelService;


    @ApiOperation("选择对象")
    @GetMapping("/obj/select/{modelId}")
    public TableDataInfo objSelect(@ApiParam(value = "模型ID", required = true) @PathVariable("modelId") Long modelId) {
        startPage();
        List<BizObj> list = bizModelService.selectObjList(modelId);
        return getDataTable(list);
    }

    @ApiOperation("添加对象")
    @GetMapping("/obj/add/{modelId}")
    public AjaxResult objAdd(@ApiParam(value = "模型ID", required = true) @PathVariable("modelId") Long modelId, @ApiParam(value = "对象ID数组", required = true) Long[] objIds) {
        return success(bizModelService.insertObj(modelId, objIds));
    }

     @ApiOperation("选择健康度指标")
    @GetMapping("/metrics/select/{modelId}")
    public TableDataInfo metricsSelect(@ApiParam(value = "模型ID", required = true) @PathVariable("modelId") Long modelId) {
        startPage();
        List<BizObj> list = bizModelService.selectObjList(modelId);
        return getDataTable(list);
    }

    @ApiOperation("添加对象")
    @GetMapping("/metrics/add/{modelId}")
    public AjaxResult metricsAdd(@ApiParam(value = "模型ID", required = true) @PathVariable("modelId") Long modelId, @ApiParam(value = "指标ID数组", required = true) Long[] metricsIds) {
        return success(bizModelService.insertObj(modelId, metricsIds));
    }



    /**
     * 查询业务模型列表
     */
    @ApiOperation("查询业务模型列表")
    @PreAuthorize("@ss.hasPermi('hl:bm:list')")
    @GetMapping("/list")
    public TableDataInfo list(BizModel bizModel) {
        startPage();
        List<BizModel> list = bizModelService.selectBizModelList(bizModel);
        return getDataTable(list);
    }

    /**
     * 导出业务模型列表
     */
    @ApiOperation("导出业务模型列表")
    @PreAuthorize("@ss.hasPermi('hl:bm:export')")
    @Log(title = "业务模型", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizModel bizModel) {
        List<BizModel> list = bizModelService.selectBizModelList(bizModel);
        ExcelUtil<BizModel> util = new ExcelUtil<BizModel>(BizModel.class);
        util.exportExcel(response, list, "业务模型数据");
    }

    /**
     * 获取业务模型详细信息
     */
    @ApiOperation("获取业务模型详细信息")
    @PreAuthorize("@ss.hasPermi('hl:bm:query')")
    @GetMapping(value = "/{modelId}")
    public AjaxResult getInfo(@PathVariable("modelId") Long modelId) {
        return success(bizModelService.selectBizModelByModelId(modelId));
    }

    /**
     * 新增业务模型
     */
    @ApiOperation("新增业务模型")
    @PreAuthorize("@ss.hasPermi('hl:bm:add')")
    @Log(title = "业务模型", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizModel bizModel) {
        return toAjax(bizModelService.insertBizModel(bizModel));
    }

    /**
     * 修改业务模型
     */
    @ApiOperation("修改业务模型")
    @PreAuthorize("@ss.hasPermi('hl:bm:edit')")
    @Log(title = "业务模型", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizModel bizModel) {
        return toAjax(bizModelService.updateBizModel(bizModel));
    }

    /**
     * 删除业务模型
     */
    @ApiOperation("删除业务模型")
    @PreAuthorize("@ss.hasPermi('hl:bm:remove')")
    @Log(title = "业务模型", businessType = BusinessType.DELETE)
    @DeleteMapping("/{modelIds}")
    public AjaxResult remove(@PathVariable Long[] modelIds) {
        return toAjax(bizModelService.deleteBizModelByModelIds(modelIds));
    }
}
